using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SequenceSynchroTrigger
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the MSX-E object
        /// </summary>
        private Msxexxxx msxe;

        /// <summary>
        /// Store the data client
        /// </summary>
        private DataClient client;

        /// <summary>
        /// Store the selected channels
        /// </summary>
        private List<int> selectedChannels;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Event handler: occurs once when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load default parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
            this.timeUnitComboBox.SelectedIndex = 0;
            this.outputModeComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Event handler: occurs once when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Event handler: click on the "Initialise connection" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initialiseConnectionButton_Click(object sender, EventArgs e)
        {
            // Check the validity of the IP address
            if (this.ipTextBox.Valid == false)
                return;

            // Update some graphics
            this.acquisitionGroupBox.Enabled = false;
            this.initialiseConnectionButton.Enabled = false;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Initialise the connection with the MSX-E system.";
            this.actionToolStripStatusLabel.Visible = true;

            // Start the connection background worker
            this.connectionBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Job of the connection background worker
        /// The connection task is done in background since it may takes some time to initialise the system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Construct the new MSX-E object
            Msxexxxx m = new Msxexxxx(this.ipTextBox.Text);

            // Do the connection
            retval = m.Connect(out error);
            e.Result = new ConnectionWorkerResult(m, retval, error);
        }

        /// <summary>
        /// Event handler: Connection background worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ConnectionWorkerResult r = (ConnectionWorkerResult)e.Result;

            // Error ?
            if (r.retval != 0)
            {
                string error = string.Empty;
                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Error occurred.";
                // Reset the system type
                this.systemInformationTextBox.Text = string.Empty;
                // Display the right text
                switch (r.retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + r.error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + r.error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                // Store the object
                this.msxe = r.msxe;

                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + r.msxe.IPAddress;
                // Display the system type
                this.systemInformationTextBox.Text = r.msxe.SystemType;

                // Graphics
                this.acquisitionGroupBox.Enabled = true;

                this.sequenceListView.Items.Clear();
                this.valuesListView.Items.Clear();

                // Fill the channels
                for (int i = 0; i < (int)this.msxe.AcquisitionChannelNb; i++)
                {
                    ListViewItem item = new ListViewItem(new string[] { string.Empty, i.ToString(), this.msxe.GetAcquisitionChannelType(i) });
                    item.Checked = true;
                    sequenceListView.Items.Add(item);

                    ListViewItem valueItem = new ListViewItem(new string[] { i.ToString(), "Waiting data..." });
                    valuesListView.Items.Add(valueItem);
                }

                // Sequence counter
                ListViewItem sequenceCounterItem = new ListViewItem(new string[] { "Counter", "Waiting data..." });
                valuesListView.Items.Add(sequenceCounterItem);

                // Timestamp second
                ListViewItem tsItem = new ListViewItem(new string[] { "Seconds", "Waiting data..." });
                valuesListView.Items.Add(tsItem);

                // Timestamp micro seconds
                ListViewItem tusItem = new ListViewItem(new string[] { "Microseconds", "Waiting data..." });
                valuesListView.Items.Add(tusItem);
            }

            // Enable the "Initialise Connection" button
            this.initialiseConnectionButton.Enabled = true;
        }

        /// <summary>
        /// Event handler: click on "Start sequence" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startSequenceButton_Click(object sender, EventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Construct the selected channels
            this.selectedChannels = new List<int>();
            for (int i = 0; i < (int)this.msxe.AcquisitionChannelNb; i++)
            {
                if (sequenceListView.Items[i].Checked)
                    this.selectedChannels.Add(i);
            }

            // Create the data-client
            this.client = new DataClient(this.msxe.IPAddress, 8989, 12 + this.selectedChannels.Count * 4);
            if (this.client.StartListening() != 0)
            {
                MessageBox.Show("An error occured while opening socket connection on port 8989.",
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            // Wait to be sure that the connection is opened
            System.Threading.Thread.Sleep(500);

            // Start the sequence
            retval = this.msxe.AcquisitionSequenceInitAndStart(this.selectedChannels, out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                client.Close();
                return;
            }

            // Graphics
            this.startSequenceButton.Enabled = false;
            this.stopSequenceButton.Enabled = true;
            this.systemConnectionGroupBox.Enabled = false;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Visible = true;
            this.actionToolStripStatusLabel.Text = "Waiting data...";

            // Start the monitoring worker
            monitoringBackgroundWorker.RunWorkerAsync(new MonitorWorkerParams(this.client, this.msxe));
        }

        /// <summary>
        /// Event handler: click on "Stop sequence" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopSequenceButton_Click(object sender, EventArgs e)
        {
            // Stop acquisition
            this.msxe.StopAll();

            // Graphics
            this.stopSequenceButton.Enabled = false;

            // Stop the background worker
            monitoringBackgroundWorker.CancelAsync();
        }

        /// <summary>
        /// Main job of the monitoring background worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitoringBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            MonitorWorkerParams p = (MonitorWorkerParams)e.Argument;
            int acquisitionCount = 0;

            // Endless loop
            for (; ; )
            {
                bool first = true;
                bool report = false;

                if (worker.CancellationPending)
                    return;

                // While data are available
                while (true)
                {
                    int count = 0;

                    // Get the first packet, flush the others
                    DataClientEventArgs a = p.client.PopAndFlush(ref count);
                    if (a == null)
                        break;

                    report = true;
                    acquisitionCount+= count;

                    // Event for graphics ?
                    if (first)
                        worker.ReportProgress(1, a);

                    first = false;
                }

                // Report number of events
                if ( report)
                    worker.ReportProgress(2, acquisitionCount);

                System.Threading.Thread.Sleep(50);
            }
        }

        /// <summary>
        /// Event handler: the monitoring background worker report some progress
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitoringBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            switch (e.ProgressPercentage)
            {
                case 1:
                    DataClientEventArgs a = (DataClientEventArgs)e.UserState;
                    uint ts = BitConverter.ToUInt32(a.data, 0);
                    uint tus = BitConverter.ToUInt32(a.data, 4);
                    uint counter = BitConverter.ToUInt32(a.data, 8);
                    this.valuesListView.Items[(int)this.msxe.AcquisitionChannelNb].SubItems[1].Text = counter.ToString();
                    this.valuesListView.Items[(int)this.msxe.AcquisitionChannelNb + 1].SubItems[1].Text = ts.ToString();
                    this.valuesListView.Items[(int)this.msxe.AcquisitionChannelNb + 2].SubItems[1].Text = tus.ToString();

                    for (int i = 0; i < this.selectedChannels.Count; i++)
                    {
                        uint value = BitConverter.ToUInt32(a.data, 12 + i * 4);
                        this.valuesListView.Items[this.selectedChannels[i]].SubItems[1].Text = value.ToString();
                    }
                    break;

                case 2:
                    int count = (int)e.UserState;
                    this.actionToolStripStatusLabel.Text = count.ToString() + " sequences performed.";
                    break;
            }
        }

        /// <summary>
        /// Event handler: the monitoring background worker is over
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitoringBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.client.Close();
            this.startSequenceButton.Enabled = true;
            this.systemConnectionGroupBox.Enabled = true;
            this.actionToolStripProgressBar.Visible = false;
            this.actionToolStripStatusLabel.Visible = false;
        }

        /// <summary>
        /// Event handler: click on "Start Synchro Timer" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startSynchroTimerButton_Click(object sender, EventArgs e)
        {
            ulong reloadValue = 0;
            ulong cycleNumber = 0;
            int retval = 0;
            string error = string.Empty;

            // Get the reload value
            try
            {
                reloadValue = Convert.ToUInt64(this.reloadValueTextBox.Text);
                if ((reloadValue < 0) || (reloadValue > 0xFFFF))
                {
                    throw new Exception("Reload value must be between 0 and 65535.");
                }
                if ((reloadValue < 5) && (timeUnitComboBox.SelectedIndex == 0))
                {
                    throw new Exception("Minimum reload value is 5 microseconds.");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("The reload value parameter is wrong: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            // Get the cycle number
            try
            {
                cycleNumber = Convert.ToUInt64(this.cycleNumberTextBox.Text);
                if ((reloadValue < 0) || (reloadValue > 0xFFFF))
                {
                    throw new Exception("Cycle number value must be between 0 and 65535.");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("The cycle number parameter is wrong: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            // Start
            retval = this.msxe.InitAndStartSynchroTimer((ulong)timeUnitComboBox.SelectedIndex,
                reloadValue,
                cycleNumber,
                (ulong)this.outputModeComboBox.SelectedIndex,
                out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Graphics
            this.startSynchroTimerButton.Enabled = false;
            this.stopSynchroTimerButton.Enabled = true;
        }

        /// <summary>
        /// Event handler: click on "Stop Synchro Timer" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopSynchroTimerButton_Click(object sender, EventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Graphics
            this.startSynchroTimerButton.Enabled = true;
            this.stopSynchroTimerButton.Enabled = false;

            // Stop
            retval = this.msxe.StopAndReleaseSynchroTimer(out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}