using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Write1OutputValueTrigger
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Web reference to the system
        /// </summary>
        private MSXExxxxWeb.MSXExxxx sys;

        /// <summary>
        /// Contains the number of available outputs
        /// </summary>
        private int anaOutCount;

        /// <summary>
        /// List of available channels
        /// </summary>
        private List<String> availableChannels;

        /// <summary>
        /// Store the polarity of the channels
        /// </summary>
        private int[] channelPolarity;

        /// <summary>
        /// Store the value written by the user
        /// </summary>
        private int[] channelValue;

        /// <summary>
        /// Currently selected index
        /// </summary>
        private int currentSelectedIndex;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.availableChannels = new List<string>();
            this.anaOutCount = -1;
            this.sys = null;
        }

        /// <summary>
        /// Occurs the first time the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Click on ok button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void okButton_Click(object sender, EventArgs e)
        {
            ulong channelNumber = 0;
            this.sys = new MSXExxxxWeb.MSXExxxx();
            this.sys.Url = "http://" + this.ipTextBox.Text + ":5555";
            this.sys.Timeout = 2000;

            // Stop display timer
            this.updateTimer.Stop();

            // default
            controlPanel.Enabled = false;
            actionPanel.Enabled = false;
            triggerPanel.Enabled = false;
            valuesListView.Enabled = false;

            // get the number of channels
            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sys.AnalogOutputGetNumberOfChannels(0, out channelNumber);
                if ((response.iReturnValue != 0) || (response.syserrno != 0))
                {
                    MessageBox.Show("AnalogOutputGetNumberOfChannels. iReturnValue : " + response.iReturnValue.ToString() + " syserrno : " + response.syserrno.ToString());
                    return;
                }
                this.anaOutCount = (int)channelNumber;

                // Update graphics
                this.anaOutCountLabel.Text = this.anaOutCount.ToString();
            }
            catch (Exception ex)
            {
                MessageBox.Show("AnalogOutputGetNumberOfChannels exception : " + ex.Message);
                return;
            }

            // create the polarity
            this.channelPolarity = new int[this.anaOutCount];
            for (int i = 0; i < this.channelPolarity.Length; i++)
            {
                this.channelPolarity[i] = 0; // unipolar
            }

            // create the value
            this.channelValue = new int[this.anaOutCount];
            for (int i = 0; i < this.channelPolarity.Length; i++)
            {
                this.channelValue[i] = 0;
            }

            // Create the database for the channel combo box
            this.availableChannels.Clear();
            for (int i = 0; i < (int)(this.anaOutCount); i++)
            {
                this.availableChannels.Add(i.ToString());
            }
            this.channelIndexComboBox.DataSource = this.availableChannels;

            // Initialise controls
            this.polarityComboBox.SelectedIndex = 0;

            // Construct the list view items
            this.valuesListView.Items.Clear();
            for (int i = 0; i < anaOutCount; i++)
            {
                ListViewItem item = new ListViewItem(new string[] { i.ToString(), "-","-" });
                this.valuesListView.Items.Add(item);
            }
            this.valuesListView.Update();

            // Enables some controls
            controlPanel.Enabled = true;
            actionPanel.Enabled = true;
            triggerPanel.Enabled = true;
            valuesListView.Enabled = true;

            // Start update timer
            this.updateTimer.Start();
        }

        /// <summary>
        /// Change the hardware trigger check box state
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void hardwareTriggerCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.risingEdgeCheckBox.Enabled = this.hardwareTriggerCheckBox.Checked;
            this.fallingEdgeCheckBox.Enabled = this.hardwareTriggerCheckBox.Checked;
            this.edgeCountTextBox.Enabled = this.hardwareTriggerCheckBox.Checked;
        }

        /// <summary>
        /// Change the value of the polarity
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void polarityComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            int value = 0;

            // security check
            if (polarityComboBox.SelectedIndex < 0)
                return;

            if (this.currentSelectedIndex < 0)
                return;

            if (this.currentSelectedIndex >= this.channelPolarity.Length)
                return;

            if (this.sys == null)
                return;

            if (this.channelPolarity == null)
                return;

            // Save the polarity
            this.channelPolarity[this.currentSelectedIndex] = this.polarityComboBox.SelectedIndex;

            // Change the range
            if (this.channelPolarity[this.currentSelectedIndex] == 0)
            {
                rangeInformationLabel.Text = "(0->32767)";
            }
            else if (this.channelPolarity[this.currentSelectedIndex] == 1)
            {
                rangeInformationLabel.Text = "(0->65535)";
            }

            try
            {
                value = Convert.ToInt32(this.digitalValueTextBox.Text);
            }
            catch (Exception)
            {
                value = 0;
            }

            if (value < 0)
                value = 0;

            if (this.channelPolarity[this.currentSelectedIndex] == 0)
            {
                if (value > 32767)
                    value = 32767;
            }
            else if (this.channelPolarity[this.currentSelectedIndex] == 1)
            {
                if (value > 65535)
                    value = 65535;
            }

            this.digitalValueTextBox.Text = value.ToString();
            UpdateVoltValue();
        }

        /// <summary>
        /// Click on the write output button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void writeButton_Click(object sender, EventArgs e)
        {
            ulong channelValue = 0;
            ulong triggerMode = 0;
            ulong edgeSelection = 0;
            ulong edgeCount = 0;

            // security check
            if (this.currentSelectedIndex < 0)
                return;

            if (this.sys == null)
                return;

            // Get the channel value
            try
            {
                channelValue = Convert.ToUInt64(this.digitalValueTextBox.Text);
                if (this.channelPolarity[this.currentSelectedIndex] == 0)
                {
                    if (channelValue > 32767)
                        channelValue = 32767;
                }
                else if (this.channelPolarity[this.currentSelectedIndex] == 1)
                {
                    if (channelValue > 65535)
                        channelValue = 65535;
                }
            }
            catch (Exception)
            {
                channelValue = 0;
            }

            // Just to be sure
            this.digitalValueTextBox.Text = channelValue.ToString();
            UpdateVoltValue();

            // construct the trigger mode
            if (hardwareTriggerCheckBox.Checked)
            {
                triggerMode += 1;
                if (risingEdgeCheckBox.Checked)
                    edgeSelection += 1;
                if (fallingEdgeCheckBox.Checked)
                    edgeSelection += 2;

                if ((risingEdgeCheckBox.Checked == false) && (fallingEdgeCheckBox.Checked == false))
                {
                    MessageBox.Show("Please choose at least rising or falling edge (or disable hardware trigger).");
                    return;
                }

                try
                {
                    edgeCount = Convert.ToUInt64(this.edgeCountTextBox.Text);
                }
                catch (Exception)
                {
                    edgeCount = 0;
                }
                if (edgeCount == 0)
                {
                    MessageBox.Show("Edge count cannot be set to 0.");
                    return;
                }
            }
            if (synchroTriggerCheckBox.Checked)
                triggerMode += 2;
            if (softwareTriggerCheckBox.Checked)
                triggerMode += 4;

            try
            {
                int syserrno = 0;
                int retval = 0;
                retval = this.sys.AnalogOutputWrite1Value((ulong)this.currentSelectedIndex,
                    0, 
                    (ulong)this.channelPolarity[this.currentSelectedIndex], 
                    triggerMode, 
                    edgeSelection, 
                    edgeCount,
                    channelValue,
                    0, 0, 0, out syserrno);
                if ((retval != 0) || (syserrno != 0))
                {
                    this.SetAsDisconnected();
                    MessageBox.Show("AnalogOutputWrite1Value. retval : " + retval.ToString() + " syserrno : " + syserrno.ToString());
                    return;
                }
            }
            catch (Exception ex)
            {
                this.SetAsDisconnected();
                MessageBox.Show("AnalogOutputWrite1Value exception : " + ex.Message);
                return;
            }            
        }

        /// <summary>
        /// Update the graphics
        /// </summary>
        private void SetAsDisconnected()
        {
            this.sys = null;
            this.currentSelectedIndex = -1;
            this.anaOutCountLabel.Text = "-";
            this.channelPolarity = null;
            this.availableChannels.Clear();
            this.channelIndexComboBox.DataSource = null;
            this.channelIndexComboBox.Update();
            controlPanel.Enabled = false;
            actionPanel.Enabled = false;
            triggerPanel.Enabled = false;
            valuesListView.Enabled = false;
            this.updateTimer.Stop();
        }

        /// <summary>
        /// Selected channel changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void channelIndexComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // security check
            if (this.channelIndexComboBox.SelectedIndex < 0)
                return;

            if (this.sys == null)
                return;

            if (this.channelPolarity == null)
                return;

            // Set the selected index
            this.currentSelectedIndex = this.channelIndexComboBox.SelectedIndex;

            if (this.currentSelectedIndex >= this.channelPolarity.Length)
                return;

            // Set the polarity
            this.polarityComboBox.SelectedIndex = this.channelPolarity[this.currentSelectedIndex];

            // Set the last written value
            this.digitalValueTextBox.Text = this.channelValue[this.currentSelectedIndex].ToString();
            UpdateVoltValue();
        }

        /// <summary>
        /// Occurs when the text is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void digitalValueTextBox_TextChanged(object sender, EventArgs e)
        {
            this.UpdateVoltValue();
        }

        /// <summary>
        /// Update the volt value
        /// </summary>
        private void UpdateVoltValue()
        {
            int value = 0;
            try
            {
                value = Convert.ToInt32(this.digitalValueTextBox.Text);
            }
            catch (Exception)
            {
                value = 0;
            }
            this.channelValue[this.currentSelectedIndex] = value;

            // Compute the value (volt)
            if (this.channelPolarity[this.currentSelectedIndex] == 0)
            {
                if (value > 32767)
                    value = 32767;

                double voltage = (((double)value * 10) / 32767);
                this.valueVoltLabel.Text = String.Format("{0:0.###}", voltage) + " V";
            }
            else if (this.channelPolarity[this.currentSelectedIndex] == 1)
            {
                if (value > 65535)
                    value = 65535;

                double voltage = ((((double)value - 32768) * 10) / 32767);
                this.valueVoltLabel.Text = String.Format("{0:0.###}", voltage) + " V";
            }
        }

        /// <summary>
        /// Update timer tick
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updateTimer_Tick(object sender, EventArgs e)
        {
            try
            {
                for (int i = 0; i < anaOutCount; i++)
                {
                    ulong channelValue = 0;
                    double voltage = 0;
                    // Get the current value of the channel

                    MSXExxxxWeb.DefaultResponse response = null;
                    response = this.sys.AnalogOutputGetChannelValue((ulong)i, 0, out channelValue);
                    if ((response.iReturnValue != 0) || (response.syserrno != 0))
                    {
                        channelValue = 0;
                    }
                    this.valuesListView.Items[i].SubItems[1].Text = channelValue.ToString();
                    voltage = ((((double)channelValue - 32768) * 10) / 32767);
                    this.valuesListView.Items[i].SubItems[2].Text = String.Format("{0:0.###}", voltage);
                }
            }
            catch (Exception)
            {
                for (int i = 0; i < anaOutCount; i++)
                {
                    this.valuesListView.Items[i].SubItems[1].Text = "-";
                }
            }
        }

        /// <summary>
        /// Click on software trigger
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void triggerButton_Click(object sender, EventArgs e)
        {
            // security check
            if (this.sys == null)
            {
                return;
            }

            // trigger the outputs
            try
            {
                int syserrno = 0;
                int retval = 0;
                retval = this.sys.AnalogOutputTriggerOutputs(0, out syserrno);
                if ((retval != 0) || (syserrno != 0))
                {
                    this.SetAsDisconnected();
                    MessageBox.Show("AnalogOutputTriggerOutputs. retval : " + retval.ToString() + " syserrno : " + syserrno.ToString());
                    return;
                }
            }
            catch (Exception ex)
            {
                this.SetAsDisconnected();
                MessageBox.Show("AnalogOutputTriggerOutputs exception : " + ex.Message);
                return;
            }
        }
    }
}