using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Diagnosis
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the MSX-E object
        /// </summary>
        private Msxexxxx msxe;

        /// <summary>
        /// Constructor of the graphical interface
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Event handler: occurs once when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load default parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Event handler: occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Event handler: click on the "Initialise connection" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initialiseConnectionButton_Click(object sender, EventArgs e)
        {
            // Check the validity of the IP address
            if (this.ipTextBox.Valid == false)
                return;

            // Update some graphics
            this.initialiseConnectionButton.Enabled = false;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Initialise the connection with the MSX-E system.";
            this.actionToolStripStatusLabel.Visible = true;

            this.digitalIOGroupBox.Enabled = false;

            // Start the connection background worker
            this.connectionBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Job of the connection background worker
        /// The connection task is done in background since it may takes some time to initialise the system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Construct the new MSX-E object
            Msxexxxx m = new Msxexxxx(this.ipTextBox.Text);

            // Do the connection
            retval = m.Connect(out error);
            e.Result = new ConnectionWorkerResult(m, retval, error);
        }

        /// <summary>
        /// Event handler: Connection background worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ConnectionWorkerResult r = (ConnectionWorkerResult)e.Result;

            // Error ?
            if (r.retval != 0)
            {
                string error = string.Empty;
                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Error occurred.";
                // Reset the system type
                this.systemInformationTextBox.Text = string.Empty;
                // Display the right text
                switch (r.retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + r.error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + r.error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                // Store the object
                this.msxe = r.msxe;

                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + r.msxe.IPAddress;
                // Display the system type
                this.systemInformationTextBox.Text = r.msxe.SystemType;

                // Update
                this.UpdateDiagnosis();

                // Enable controls
                this.digitalIOGroupBox.Enabled = true;
            }

            // Enable the "Initialise Connection" button
            this.initialiseConnectionButton.Enabled = true;
        }

        /// <summary>
        /// Update the list view
        /// </summary>
        private void UpdateDiagnosis()
        {
            int currentIndex = 0;

            // Clear
            this.digitalIOListView.Items.Clear();

            // Loop through all the possibles ports
            for (int i = 0; i < (int)this.msxe.DigIOPortNb; i++)
            {
                ulong shortCircuit = 0;
                ulong powerSupply = 0;
                DigitalIOPort port = this.msxe.GetDigitalIoPortInfo(i);
                int retval = 0;
                string error = string.Empty;

                // Get short circuit state of the port
                retval = this.msxe.DigitalIOTestOutputsShortCircuit((ulong)i, out shortCircuit, out error);
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }

                // Get power supply state of the port
                retval = this.msxe.DigitalIOTestOutputsPowerSupply((ulong)i, out powerSupply, out error);
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }

                Console.WriteLine("Short-Circuit: " + shortCircuit.ToString("X"));
                Console.WriteLine("Power supply: " + powerSupply.ToString("X"));

                
                // Assign graphics
                for (int j = 0; j < (int)port.ChannelNumber; j++)
                {
                    bool sc = false;
                    bool ps = false;

                    if (((shortCircuit >> j) & 1) == 1)
                    {
                        sc = true;
                    }

                    if (((powerSupply >> j) & 1) == 1)
                    {
                        ps = true;
                    }

                    ListViewItem item = new ListViewItem(new string[] { currentIndex.ToString(),
                        sc == true ? "Short-Circuit!" : "OK",
                        ps == true ? "No power supply!" : "OK",
                        i.ToString()});

                    // Short-circuit
                    if (sc == true)
                    {
                        item.SubItems[1].BackColor = Color.Red;
                    }
                    else
                    {
                        item.SubItems[1].BackColor = Color.LightGreen;
                    }

                    // Power supply
                    if (ps == true)
                    {
                        item.SubItems[2].BackColor = Color.Red;
                    }
                    else
                    {
                        item.SubItems[2].BackColor = Color.LightGreen;
                    }
                    
                    item.UseItemStyleForSubItems = false;
                    digitalIOListView.Items.Add(item);
                    currentIndex++;
                }
            }
        }

        /// <summary>
        /// Event handler: click on the "Refresh" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void refreshButton_Click(object sender, EventArgs e)
        {
            this.UpdateDiagnosis();
        }

        /// <summary>
        /// Event handler: click on the "Rearm" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rearmButton_Click(object sender, EventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            retval = this.msxe.DigitalIORearmOutputsShortCircuit(out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }

                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Display success message
            MessageBox.Show("Rearm was succesful",
                "Information",
                MessageBoxButtons.OK,
                MessageBoxIcon.Information);
        }
    }
}