using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Database
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Database object
        /// </summary>
        private TransducerDatabase database;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Occurs the first time the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;

            // Default
            this.frequencyComboBox.SelectedIndex = 0;
            this.typeComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Click on initialise button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initButton_Click(object sender, EventArgs e)
        {
            this.initButton.Enabled = false;

            // Create the database
            MSXExxxx mx = new MSXExxxx(this.ipTextBox.Text);
            this.database = new TransducerDatabase(mx);

            this.ReadSystemDatabase();
        }

        /// <summary>
        /// Read the database of the system
        /// </summary>
        private void ReadSystemDatabase()
        {
            // Update some graphics
            this.statusToolStripStatusLabel.Text = "Reading database from the system.";
            this.inProgressToolStripProgressBar.Visible = true;

            // Start the worker to get the database
            getDatabaseBackgroundWorker.RunWorkerAsync(this.database);            
        }

        /// <summary>
        /// Update
        /// </summary>
        private void UpdateTransducersListView()
        {
            // Clear the items
            this.transducerListView.Items.Clear();

            // Add the items
            foreach (TransducerDatabaseItem i in this.database.Items)
            {
                this.transducerListView.Items.Add(i.ConstructListViewItem());
            }
        }

        /// <summary>
        /// Get the content of the database
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getDatabaseBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            TransducerDatabase mxdatabase = (TransducerDatabase)e.Argument;
            string error = string.Empty;

            // Get the database
            if (mxdatabase.LoadFromSystem(ref error) != 0)
            {
                worker.ReportProgress(1, error);
                return;
            }

            // Success
            worker.ReportProgress(99);
        }

        /// <summary>
        /// Get something from the database getter worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getDatabaseBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Text message ?
            if (e.ProgressPercentage == 1)
            {
                this.AddLog((string)e.UserState);
                return;
            }
            // Success
            if (e.ProgressPercentage == 99)
            {
                this.UpdateTransducersListView();
                this.addButton.Enabled = true;
            }
        }

        /// <summary>
        /// Add a log
        /// </summary>
        /// <param name="log"></param>
        private void AddLog(String log)
        {
            this.logTextBox.Text += DateTime.Now.ToString() + "\t" + log + Environment.NewLine;
            this.logTextBox.SelectionStart = this.logTextBox.Text.Length;
            this.logTextBox.ScrollToCaret();
        }

        /// <summary>
        /// Database getter worker is finished
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getDatabaseBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // Update some graphics
            this.statusToolStripStatusLabel.Text = string.Empty;
            this.inProgressToolStripProgressBar.Visible = false;
        }

        /// <summary>
        /// Click on the delete button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void deleteButton_Click(object sender, EventArgs e)
        {
            string error = string.Empty;

            if (transducerListView.SelectedItems.Count <= 0)
            {
                this.AddLog("Select a transducer and retry.");
                return;
            }

            // Update some graphics
            this.deleteButton.Enabled = false;
            this.statusToolStripStatusLabel.Text = "Deleting transducer.";
            this.inProgressToolStripProgressBar.Visible = true;

            // Start the worker
            deleteDatabaseBackgroundWorker.RunWorkerAsync(new DeleteWorkerParam(this.database, transducerListView.SelectedIndices[0]));
        }

        /// <summary>
        /// Selected index of the transducer list view is changed
        /// We update the value of the add fields
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void transducerListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            // security
            if (transducerListView.SelectedItems.Count <= 0)
                return;

            // get selected item
            int index = transducerListView.SelectedIndices[0];

            // get the item and set the values of the fields
            TransducerDatabaseItem item = this.database.Items[index];
            this.indexTextBox.Text = item.index.ToString();
            this.nameTextBox.Text = item.name;
            this.rangeTextBox.Text = item.range.ToString();
            switch (item.frequency)
            {
                case 5000:
                    this.frequencyComboBox.SelectedIndex = 0;
                    break;
                case 10000:
                    this.frequencyComboBox.SelectedIndex = 1;
                    break;
                case 12500:
                    this.frequencyComboBox.SelectedIndex = 2;
                    break;
                case 20000:
                    this.frequencyComboBox.SelectedIndex = 3;
                    break;
            }
            switch (item.type)
            {
                case 0:
                    this.typeComboBox.SelectedIndex = 0;
                    break;
                case 1:
                    this.typeComboBox.SelectedIndex = 1;
                    break;
                case 2:
                    this.typeComboBox.SelectedIndex = 2;
                    break;
                case 3:
                    this.typeComboBox.SelectedIndex = 3;
                    break;
            }
            this.veffTextBox.Text = item.veff.ToString();
            this.sensitivityTextBox.Text = item.sensitivity.ToString();
            this.impedanceTextBox.Text = item.impedance.ToString();
        }

        /// <summary>
        /// Click on add
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void addButton_Click(object sender, EventArgs e)
        {
            ulong index = 0;
            string name = string.Empty;
            ulong type = 0; // 0: HB, 1: LVDT, 2: MAHR-HB, 3: MAHR-LVDT
            double range = 0;
            ulong frequency = 0;
            double veff = 0;
            double sensitivity = 0;
            ulong impedance = 0 ;
            string error = string.Empty;

            try
            {
                index = Convert.ToUInt64(this.indexTextBox.Text);
                name = this.nameTextBox.Text;
                type = (ulong)this.typeComboBox.SelectedIndex;
                range = Convert.ToDouble(this.rangeTextBox.Text);
                switch (this.frequencyComboBox.SelectedIndex)
                {
                    case 0: frequency = 5000; break;
                    case 1: frequency = 10000; break;
                    case 2: frequency = 12500; break;
                    case 3: frequency = 20000; break;
                }
                veff = Convert.ToDouble(this.veffTextBox.Text);
                sensitivity = Convert.ToDouble(this.sensitivityTextBox.Text);
                impedance = Convert.ToUInt64(this.impedanceTextBox.Text);
            }
            catch (Exception ex)
            {
                this.AddLog("One of the sensor caracteristic is wrong: " + ex.Message);
                return;
            }

            // Update some graphics
            this.addButton.Enabled = false;
            this.statusToolStripStatusLabel.Text = "Adding transducer.";
            this.inProgressToolStripProgressBar.Visible = true;

            // Call the worker (since added a transducer inside the database takes a lot of time)
            addDatabaseBackgroundWorker.RunWorkerAsync(new AddWorkerParams(new TransducerDatabaseItem(index, name, type, range, frequency, veff, sensitivity, impedance), this.database));
        }

        /// <summary>
        /// Add a transducer inside the database
        /// Make it inside a worker since it needs a lot of time
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void addDatabaseBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            AddWorkerParams p = (AddWorkerParams)e.Argument;
            string error = string.Empty;

            // Add the item
            if (p.database.AddTransducer(p.item, ref error) != 0)
            {
                worker.ReportProgress(1, error);
                return;
            }

            // Success
            worker.ReportProgress(99);
        }

        /// <summary>
        /// Add worker is over
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void addDatabaseBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.addButton.Enabled = true;
        }

        /// <summary>
        /// Add worker report information
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void addDatabaseBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Error ?
            if (e.ProgressPercentage == 1)
            {
                this.statusToolStripStatusLabel.Text = string.Empty;
                this.inProgressToolStripProgressBar.Visible = false;
                this.AddLog((string)e.UserState);
                return;
            }
            // Success
            if (e.ProgressPercentage == 99)
            {
                this.AddLog("Transducer succesfully added. Refresh database.");
                this.ReadSystemDatabase();
            }
        }

        /// <summary>
        /// Delete a transducer from the database
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void deleteDatabaseBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            DeleteWorkerParam p = (DeleteWorkerParam)e.Argument;
            string error = string.Empty;

            // Delete the item
            if (p.database.RemoveItemByIndex(p.index, ref error) != 0)
            {
                worker.ReportProgress(1, error);
                return;
            }

            // Success
            worker.ReportProgress(99);
        }

        /// <summary>
        /// Delete worker is over
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void deleteDatabaseBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.deleteButton.Enabled = true;
        }

        /// <summary>
        /// Delete worker report information
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void deleteDatabaseBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Error ?
            if (e.ProgressPercentage == 1)
            {
                this.statusToolStripStatusLabel.Text = string.Empty;
                this.inProgressToolStripProgressBar.Visible = false;
                this.AddLog((string)e.UserState);
                return;
            }
            // Success
            if (e.ProgressPercentage == 99)
            {
                this.AddLog("Transducer succesfully deleted. Refresh database.");
                this.ReadSystemDatabase();
            }
        }
    }
}